using System;
using System.Collections.Generic;
using System.Text;
using Infragistics.Win;
using Infragistics.Win.UltraWinEditors;

namespace WinGrid___Different_Formats_in_the_Same_Column_CS
{    
    public class EditorCache : IDisposable
    {
        Dictionary<string, EmbeddableEditorBase> editorsDictionary;

        public EmbeddableEditorBase GetEditor(string format)
        {
            // Try to get an existing, cached editor. 
            EmbeddableEditorBase editor = this.GetExistingEditor(format);
            if (editor != null)
                return editor;            

            // If we failed to find an existing editor, create a new one and 
            // cache it. 
            return this.CreateNewEditor(format);            
        }

        // Try to find an exiting editor in the cache. 
        private EmbeddableEditorBase GetExistingEditor(string format)
        {
            // If the editorsDictionary is null, nothing is cached, so 
            // just return null. 
            if (this.editorsDictionary == null)
                return null;

            // Try to find an editor with the specified format. 
            EmbeddableEditorBase editor;
            bool success = this.editorsDictionary.TryGetValue(format, out editor);

            // If we found an existing editor, return it, otherwise, return null.
            return success ? editor : null;
        }

        private EmbeddableEditorBase CreateNewEditor(string format)
        {
            // Create a new DefaultEditorOwnerSettings and set the format. 
            DefaultEditorOwnerSettings settings = new DefaultEditorOwnerSettings();
            settings.Format = format;

            // Create a new DefaultEditorOwner and pass in the settings. 
            DefaultEditorOwner editorOwner = new DefaultEditorOwner(settings);

            // Create a new editor with the owner. 
            EmbeddableEditorBase editor = new EditorWithText(editorOwner);

            // Cache the editor
            if (this.editorsDictionary == null)
                this.editorsDictionary = new Dictionary<string, EmbeddableEditorBase>();            

            this.editorsDictionary.Add(format, editor);

            // return the new editor. 
            return editor;            
        }

        #region IDisposable Members

        public void Dispose()
        {
            if (this.editorsDictionary != null)
            {
                EmbeddableEditorBase[] editors = new EmbeddableEditorBase[this.editorsDictionary.Count];
                this.editorsDictionary.Values.CopyTo(editors, 0);
                foreach (EmbeddableEditorBase editor in editors)
                    editor.Dispose();
            }
        }

        #endregion
    }
}
